@@ -34,6 +34,7 @@ from fidesops.api.v1.scope_registry import (
 from fidesops.api.v1.urn_registry import (
     PRIVACY_REQUEST_APPROVE,
     PRIVACY_REQUEST_DENY,
+    PRIVACY_REQUEST_MANUAL_ERASURE,
     PRIVACY_REQUEST_MANUAL_INPUT,
     PRIVACY_REQUEST_RESUME,
     REQUEST_PREVIEW,
@@ -47,7 +48,7 @@ from fidesops.models.audit_log import AuditLog, AuditLogAction
 from fidesops.models.client import ClientDetail
 from fidesops.models.connectionconfig import ConnectionConfig
 from fidesops.models.datasetconfig import DatasetConfig
-from fidesops.models.policy import ActionType, Policy, PolicyPreWebhook
+from fidesops.models.policy import PausedStep, Policy, PolicyPreWebhook
 from fidesops.models.privacy_request import (
     ExecutionLog,
     PrivacyRequest,
@@ -64,6 +65,7 @@ from fidesops.schemas.privacy_request import (
     PrivacyRequestResponse,
     PrivacyRequestVerboseResponse,
     ReviewPrivacyRequestIds,
+    RowCountRequest,
 )
 from fidesops.service.privacy_request.request_runner_service import PrivacyRequestRunner
 from fidesops.service.privacy_request.request_service import (
@@ -577,23 +579,14 @@ def resume_privacy_request(
 
 
 def validate_manual_input(
-    manual_rows: List[Row], collection: CollectionAddress, db: Session
+    manual_rows: List[Row],
+    collection: CollectionAddress,
+    dataset_graph: DatasetGraph,
 ) -> None:
     """Validate manually-added data for a collection.
 
     The specified collection must exist and all fields must be previously defined.
     """
-    datasets = DatasetConfig.all(db=db)
-    dataset_graphs = [dataset_config.get_graph() for dataset_config in datasets]
-    dataset_graph = DatasetGraph(*dataset_graphs)
-
-    node: Optional[Node] = dataset_graph.nodes.get(collection)
-    if not node:
-        raise HTTPException(
-            status_code=HTTP_422_UNPROCESSABLE_ENTITY,
-            detail=f"Cannot save manual rows. No collection in graph with name: '{collection.value}'.",
-        )
-
     for row in manual_rows:
         for field_name in row:
             if not dataset_graph.nodes[collection].contains_field(
@@ -605,6 +598,81 @@ def validate_manual_input(
                 )
 
 
+def resume_privacy_request_with_manual_input(
+    privacy_request_id: str,
+    db: Session,
+    cache: FidesopsRedis,
+    expected_paused_step: PausedStep,
+    manual_rows: List[Row] = [],
+    manual_count: Optional[int] = None,
+) -> PrivacyRequest:
+    """Resume privacy request after validating and caching manual data for an access or an erasure request."""
+    privacy_request: PrivacyRequest = get_privacy_request_or_error(
+        db, privacy_request_id
+    )
+    if privacy_request.status != PrivacyRequestStatus.paused:
+        raise HTTPException(
+            status_code=HTTP_400_BAD_REQUEST,
+            detail=f"Invalid resume request: privacy request '{privacy_request.id}' "
+            f"status = {privacy_request.status.value}. Privacy request is not paused.",
+        )
+
+    paused_step: Optional[PausedStep]
+    paused_collection: Optional[CollectionAddress]
+    paused_step, paused_collection = privacy_request.get_paused_step_and_collection()
+    if not paused_collection or not paused_step:
+        raise HTTPException(
+            status_code=HTTP_400_BAD_REQUEST,
+            detail=f"Cannot resume privacy request '{privacy_request.id}'; no paused collection or no paused step.",
+        )
+
+    if paused_step != expected_paused_step:
+        raise HTTPException(
+            status_code=HTTP_400_BAD_REQUEST,
+            detail=f"Collection '{paused_collection}' is paused at the {paused_step.value} step. Pass in manual data instead to "
+            f"'{PRIVACY_REQUEST_MANUAL_ERASURE if paused_step == PausedStep.erasure else PRIVACY_REQUEST_MANUAL_INPUT}' to resume.",
+        )
+
+    datasets = DatasetConfig.all(db=db)
+    dataset_graphs = [dataset_config.get_graph() for dataset_config in datasets]
+    dataset_graph = DatasetGraph(*dataset_graphs)
+
+    node: Optional[Node] = dataset_graph.nodes.get(paused_collection)
+    if not node:
+        raise HTTPException(
+            status_code=HTTP_422_UNPROCESSABLE_ENTITY,
+            detail=f"Cannot save manual data. No collection in graph with name: '{paused_collection.value}'.",
+        )
+
+    if paused_step == PausedStep.access:
+        validate_manual_input(manual_rows, paused_collection, dataset_graph)
+        logger.info(
+            f"Caching manual input for privacy request '{privacy_request_id}', collection: '{paused_collection}'"
+        )
+        privacy_request.cache_manual_input(paused_collection, manual_rows)
+
+    elif paused_step == PausedStep.erasure:
+        logger.info(
+            f"Caching manually erased row count for privacy request '{privacy_request_id}', collection: '{paused_collection}'"
+        )
+        privacy_request.cache_manual_erasure_count(paused_collection, manual_count)
+
+    logger.info(
+        f"Resuming privacy request '{privacy_request_id}', {paused_step.value} step, from collection "
+        f"'{paused_collection.value}'"
+    )
+
+    privacy_request.status = PrivacyRequestStatus.in_processing
+    privacy_request.save(db=db)
+
+    PrivacyRequestRunner(
+        cache=cache,
+        privacy_request=privacy_request,
+    ).submit(from_step=paused_step)
+
+    return privacy_request
+
+
 @router.post(
     PRIVACY_REQUEST_MANUAL_INPUT,
     status_code=HTTP_200_OK,
@@ -618,50 +686,47 @@ def resume_with_manual_input(
     *,
     db: Session = Depends(deps.get_db),
     cache: FidesopsRedis = Depends(deps.get_cache),
-    manual_rows: List[Optional[Row]],
+    manual_rows: List[Row],
 ) -> PrivacyRequestResponse:
     """Resume a privacy request by passing in manual input for the paused collection.
 
     If there's no manual data to submit, pass in an empty list to resume the privacy request.
     """
-    privacy_request: PrivacyRequest = get_privacy_request_or_error(
-        db, privacy_request_id
-    )
-    if privacy_request.status != PrivacyRequestStatus.paused:
-        raise HTTPException(
-            status_code=HTTP_400_BAD_REQUEST,
-            detail=f"Invalid resume request: privacy request '{privacy_request.id}' "
-            f"status = {privacy_request.status.value}. Privacy request is not paused.",
-        )
-
-    paused_step: Optional[ActionType]
-    paused_collection: Optional[CollectionAddress]
-    paused_step, paused_collection = privacy_request.get_paused_step_and_collection()
-    if not paused_collection:
-        raise HTTPException(
-            status_code=HTTP_400_BAD_REQUEST,
-            detail=f"Cannot resume privacy request '{privacy_request.id}'; no paused collection.",
-        )
-
-    validate_manual_input(manual_rows, paused_collection, db)
-    logger.info(
-        f"Caching manual input for privacy request '{privacy_request_id}', collection: '{paused_collection}'"
-    )
-    privacy_request.cache_manual_input(paused_collection, manual_rows)
-
-    logger.info(
-        f"Resuming privacy request '{privacy_request_id}', {paused_step.value} step, from collection '{paused_collection.value}'"
-    )
-
-    privacy_request.status = PrivacyRequestStatus.in_processing
-    privacy_request.save(db=db)
-
-    PrivacyRequestRunner(
+    return resume_privacy_request_with_manual_input(
+        privacy_request_id=privacy_request_id,
+        db=db,
         cache=cache,
-        privacy_request=privacy_request,
-    ).submit(from_step=paused_step)
+        expected_paused_step=PausedStep.access,
+        manual_rows=manual_rows,
+    )
 
-    return privacy_request
+
+@router.post(
+    PRIVACY_REQUEST_MANUAL_ERASURE,
+    status_code=HTTP_200_OK,
+    response_model=PrivacyRequestResponse,
+    dependencies=[
+        Security(verify_oauth_client, scopes=[PRIVACY_REQUEST_CALLBACK_RESUME])
+    ],
+)
+def resume_with_erasure_confirmation(
+    privacy_request_id: str,
+    *,
+    db: Session = Depends(deps.get_db),
+    cache: FidesopsRedis = Depends(deps.get_cache),
+    manual_count: RowCountRequest,
+) -> PrivacyRequestResponse:
+    """Resume the erasure portion of privacy request by passing in the number of rows that were manually masked.
+
+    If no rows were masked, pass in a 0 to resume the privacy request.
+    """
+    return resume_privacy_request_with_manual_input(
+        privacy_request_id=privacy_request_id,
+        db=db,
+        cache=cache,
+        expected_paused_step=PausedStep.erasure,
+        manual_count=manual_count.row_count,
+    )
 
 
 def review_privacy_request(
